import copy
import logging
import json
import copy
import cv2

from foglamp.common import logger
import filter_ingest

import numpy as np

__author__ = "Bryce Kroencke"
__copyright__ = "Copyright (c) 2023 Dianomic Systems"
__license__ = "Apache 2.0"
__version__ = "${VERSION}"

_LOGGER = logger.setup(__name__, level=logging.DEBUG)

PLUGIN_NAME = "image-bounding-box"

_DEFAULT_CONFIG = {
    'plugin': {
        'description': 'Crop out or draw bounding boxes on an image',
        'type': 'string',
        'default': PLUGIN_NAME,
        'readonly': 'true'
    },
    'enable': {
        'description': 'Enable plugin',
        'type': 'boolean',
        'default': 'true',
        'displayName': 'Enabled',
        'order': "1"
    },
    'datapoint': {
        'description': 'Data Point containing image data to write on',
        'type': 'string',
        'default': 'img',
        'displayName': 'Image Data Point',
        'order': "2"
    },
    'forward_original': {
        'description': 'If false, the original image will be stripped from the readings leaving only the bounding box image/images',
        'type': 'boolean',
        'default': 'true',
        'displayName': 'Forward Original Image',
        'order': "3"
    },
    'box_datapoint': {
        'description': 'Data point containing the bounding boxes from an ML Inference',
        'type': 'string',
        'default': 'boxes',
        'displayName': 'Bounding Box Data Point',
        'order': "4"
    },
    'box_operation': {
        'description': 'Specifies if the bounding boxes should be cropped or drawn on to the original image',
        'type': 'enumeration',
        'default': 'Crop Bounding Boxes',
        'options': ['Crop Bounding Boxes', 'Draw Bounding Boxes'],
        'order': '5',
        'displayName': 'Bounding Box Operation'
    }
}



def plugin_info():
    """ Returns information about the plugin
    Args:
    Returns:
        dict: plugin information
    Raises:
    """
    return {
        'name': PLUGIN_NAME,
        'version': '2.4.0',
        'mode': "none",
        'type': 'filter',
        'interface': '1.0',
        'config': _DEFAULT_CONFIG
    }



def plugin_init(config, ingest_ref, callback):
    """ Initialise the plugin
    Args:
        config:     JSON configuration document for the Filter plugin configuration category
        ingest_ref: filter ingest reference
        callback:   filter callback
    Returns:
        data:       JSON object to be used in future calls to the plugin
    Raises:
    """
    data = copy.deepcopy(config)
    data['callback'] = callback
    data['ingest_ref'] = ingest_ref

    return data



def plugin_reconfigure(handle, new_config):
    """ Reconfigures the plugin
    Args:
        handle:     handle returned by the plugin initialisation call
        new_config: JSON object representing the new configuration category for the category
    Returns:
        new_handle: new handle to be used in the future calls
    """
    _LOGGER.info("Old config {} \n new config {} for {} plugin ".format(handle, new_config, PLUGIN_NAME))
    new_handle = copy.deepcopy(new_config)
    new_handle['callback'] = handle['callback']
    new_handle['ingest_ref'] = handle['ingest_ref']

    return new_handle



def plugin_shutdown(handle):
    """ Shutdowns the plugin doing required cleanup.
    Args:
        handle: handle returned by the plugin initialisation call
    Returns:
    """
    handle['callback'] = None
    handle['ingest_ref'] = None
    _LOGGER.info('{} filter plugin shutdown.'.format(PLUGIN_NAME))

def crop_box(img, box, mode):
    ymin_model = round(box[0], 3)
    xmin_model = round(box[1], 3)
    ymax_model = round(box[2], 3)
    xmax_model = round(box[3], 3)

    # map the bounding boxes from model to the window
    ymin = int(max(0, (ymin_model * img.shape[0])))
    xmin = int(max(0, (xmin_model * img.shape[1])))
    ymax = int(min(img.shape[0], (ymax_model * img.shape[0])))
    xmax = int(min(img.shape[1], (xmax_model * img.shape[1])))
    if mode == 'Crop Bounding Boxes':
        return img[ymin:ymax, xmin:xmax, :].astype(np.uint8)
    else:
        return cv2.rectangle(img, (xmin, ymin), (xmax, ymax), (10, 255, 0), 2)

def plugin_ingest(handle, data):
    """ Modify readings data and pass it onward
    Args:
        handle: handle returned by the plugin initialisation call
        data:   readings data
    """
    if handle['enable']['value'] == 'false':
        # Filter not enabled, just pass data onwards
        filter_ingest.filter_ingest_callback(handle['callback'],  handle['ingest_ref'], data)
        return


    mode = handle['box_operation']['value']

    if mode == 'Crop Bounding Boxes':
        count = 0
        # Filter is enabled: compute for each reading
        for elem in data:
            boxes = json.loads(elem["readings"][handle["box_datapoint"]["value"]])
            img = elem["readings"][handle["datapoint"]["value"]].copy()
            if handle['forward_original']['value'] == 'false':
                del elem["readings"][handle["datapoint"]["value"]]


            if isinstance(boxes, list):
                if all(isinstance(box, list) for box in boxes):
                    _LOGGER.debug("boxes is a list of lists")
                    for box in boxes:
                        elem["readings"]["cropped_"+str(count)] = crop_box(img, box, mode)
                        count+=1
                else:
                    _LOGGER.debug("boxes is a list")
                    elem["readings"]["cropped_"+str(count)] = crop_box(img, box, mode)
                    count+=1
            else:
                _LOGGER.debug("boxes is not a list")
    else:
        for elem in data:
            boxes = json.loads(elem["readings"][handle["box_datapoint"]["value"]])
            img = elem["readings"][handle["datapoint"]["value"]].copy()
            if handle['forward_original']['value'] == 'false':
                del elem["readings"][handle["datapoint"]["value"]]
            if isinstance(boxes, list):
                if all(isinstance(box, list) for box in boxes):
                    _LOGGER.debug("boxes is a list of lists")
                    for box in boxes:
                        img = crop_box(img, box, mode)
                    elem["readings"][handle["datapoint"]["value"]+"_boundingbox"] = img


                else:
                    _LOGGER.debug("boxes is a list")
                    elem["readings"][handle["datapoint"]["value"]+"_boundingbox"] = crop_box(img, box, mode)
            else:
                _LOGGER.debug("boxes is not a list")

    # Pass data onwards
    filter_ingest.filter_ingest_callback(handle['callback'],  handle['ingest_ref'], data)

    _LOGGER.debug("{} filter ingest done".format(PLUGIN_NAME))
